// jQuery.naturalWidth / jQuery.naturalHeight plugin for (already-loaded) images

// Triple-licensed: Public Domain, MIT and WTFPL license - share and enjoy!
(function($) {
	function img(url) {
		var i = new Image;
		i.src = url;
		return i;
	}

	if ('naturalWidth' in (new Image)) {
		$.fn.naturalWidth = function() {
			return this[0].naturalWidth;
		};
		$.fn.naturalHeight = function() {
			return this[0].naturalHeight;
		};
		return;
	}
	$.fn.naturalWidth = function() {
		return img(this[0].src).width;
	};
	$.fn.naturalHeight = function() {
		return img(this[0].src).height;
	};
})(jQuery);

/** 
 * Namespace for the imagetracker module.
 * @namespace Imagetracker
 */
var Imagetracker = Imagetracker || {};

/**
 * Imagetracker JS-Module adds tagging functionality to an image. 
 * Faces are tracked automatically onload and existing tags are displayed. 
 * Users with tagging rights can switch to edit mode and tag users from a pre-filtered list. 
 * Users without tagging rights can still delete tags of themselves.
 * Drawing tags manually is supported. 
 * Tags are saved to the database by clicking the save button.
 */
Imagetracker.Module = function() {
	"use strict";

	// VARIABLES
	var user,
		threshold; // threshold for pixel coordinates

	// METHODS
	var findTagByCoordinates,
		loadTags,
		plotRectangle,
		addLoadEvent,
		save,
		saveBtnListener,
		editBtnListener,
		deleteBtnListener,
		cancelBtnListener,
		mouseImgListener,
		init,
		initUser,
		getImgScaleX,
		getImgScaleY,
		createInput,
		createRect,
		convertClickCoordToImgCoord;

	threshold = 10; // threshold for pixel coordinates

	/**
	 * Finds a tag with certain coordinates within an array. 
	 * The coordinates are within a threshold to handle varying coordinates in different browsers.
	 * Returns the found tag or null.	 * 
	 * @param  {Array} array - Array of tags.
	 * @param  {Number} x     - X-coordinate to be found.
	 * @param  {Number} y     - Y-coordinate to be found.
	 * @return {Object|null}       - Tag that matches the coordinates, null if not found.
	 */
	findTagByCoordinates = function(array, x, y) {
		for (var i = 0, len = array.length; i < len; i++) {
			if ((parseInt(array[i].coord_x) - threshold) < x && x < (parseInt(array[i]
					.coord_x) + threshold)) {
				if ((parseInt(array[i].coord_y) - threshold) < y && y < (parseInt(array[i]
						.coord_y) + threshold)) {
					return array[i]; // Return as soon as the object is found
				}
			}
		}
		return null;
	};

	/**
	 * Loads tags on a specified image.
	 * @param  {Number} tagId - id of the used tag type in cms.
	 * @return {void}       
	 */
	loadTags = function(tagId) {

		var img = $('#imgtracker-' + tagId + " img");

		/**
		 * Start face detection of image.
		 */
		img.faceDetection({
			interval: 4,
			async: true,
			complete: function(faces) {
				completed(faces);
				/**
				 * Gets existing tags from the server.
				 * Creates tags (rectangle + input field) for all tracked faces 
				 * and sets the value of input field if tracked face coordinate 
				 * matches an existing tag, otherwise value is null and tag 
				 * is only displayed in edit mode. 
				 * 
				 * Creates manually drawn tags. Manual tags can be removed through a delete button. 
				 * In order to delete a tag from database, the save button has to be clicked.
				 * @param  {Array} faces - Array of objects representing tracked faces.
				 * @return {Void}  
				 */
				function completed(faces) {
					jQuery.ajax({
							url: '/imagetracker/imageTracker/getTagsByTagId',
							data: {
								tagId: tagId
							}

						})
						.done(function(response) {
							var tags = JSON.parse(response);

							var marg = 10;

							// draw rect for each tracked face
							for (var i = 0; i < faces.length; i++) {

								// position on img src
								var x = Math.floor(faces[i].x),
									y = Math.floor(faces[i].y),
									// position on img src
									left = (faces[i].x - marg),
									top = ((faces[i].y - marg)),
									width = (faces[i].width + (marg * 2)),
									height = (faces[i].height + (marg * 2));

								// position in % on current image size
								left = left * faces[i].scaleX * 100 / img[0].width;
								top = top * faces[i].scaleY * 100 / img[0].height;
								width = width * faces[i].scaleX * 100 / img[0].width;
								height = height * faces[i].scaleY * 100 / img[0].height;

								var tag, rect, input;
								tag = findTagByCoordinates(tags, x, y);
								rect = createRect(x, y);
								input = createInput(tag, false);

								rect.insertAfter(img[0]);
								rect.append(input);

								// Set dimension of rectangle
								rect.css({
									'left': left + '%',
									'top': top + '%',
									'width': width + '%',
									'height': height + '%'
								});
								// hide untagged rectangles
								if (tag === null) {
									rect.css('display', 'none');
								}
							}

							// draw manually created tags
							var scaleX, scaleY;

							scaleX = getImgScaleX(img);
							scaleY = getImgScaleY(img);

							$.each(tags, function(key, tag) {
								if (tag.manual_width) {
									var rect, input, deleteRect, side;

									rect = createRect(tag.coord_x, tag.coord_y);
									input = createInput(tag, false);
									rect.attr('data-manual', true);
									deleteRect = $(
										'<button type="button" class="rm-manual">x</button>');

									deleteRect.on('click', function(e) {
										$(this).parent().remove();
									});

									rect.append(deleteRect);
									rect.insertAfter(img[0]);
									rect.append(input);

									// manual width is in % 
									side = (tag.manual_width * scaleX);

									// Set dimension of rectangle
									rect.css({
										'left': (tag.coord_x * scaleX) * 100 / img[0].width + '%',
										'top': (tag.coord_y * scaleY) * 100 / img[0].height + '%',
										'width': side * 100 / img[0].width + '%',
										'height': side * 100 / img[0].height + '%'
									});
								}
							});

							// display edit button after tags are created
							$('.imgtracker-btns').css('display', 'inline');

							return true;

						})
						.fail(function() {
							return null;
						});

				}

			},
			error: function(code, message) {
				return null;
			}
		});
	};

	/**
	 * Gets the x-scale of an image.
	 * @param  {Object} img - Image.
	 * @return {Number}     - X-scale of image in browser.
	 */
	getImgScaleX = function(img) {
		return img[0].width / img.naturalWidth();
	};

	/**
	 * Gets the y-scale of an image.
	 * @param  {Object} img - Image.
	 * @return {Number}     - Y-scale of image in browser.
	 */
	getImgScaleY = function(img) {
		return img[0].height / img.naturalHeight();
	};

	/**
	 * Creates input field with autocomplete functionality. Adds value if tag exists.
	 * @param  {Object} tag    		- Tag.
	 * @param  {Boolean} enabled 	- True, if input field should be enabled.
	 * @return {Void}
	 */
	createInput = function(tag, enabled) {

		var input, uiWidget;
		input = $('<input>');
		input.prop('disabled', !enabled);

		uiWidget = $('<div class="ui-widget"><label for="tags"></label></div>');

		// fill in existing tags
		if (tag) {
			input.val(tag.value);
			input.attr('data-user-id', tag.user_id);

			// if current user has no tagging rights, he should be able to delete at least tags of himself
			if (user.tagging_rights === '0' && tag.user_id == user.id) {
				var removeBtn = $(
					'<button class="imgtracker-delete-btn" type="button">x</button>');
				removeBtn.attr('data-usertag-id', tag.id);
				uiWidget.append(removeBtn);
			}
		}

		uiWidget.append(input);

		/** 
		 * Enables users with tagging rights to quickly find and select from a pre-populated list 
		 * of values as they type.
		 */
		if (user.tagging_rights === '1') {
			input.autocomplete({
				autoFocus: true,
				/**
				 * Defines the data to use. Connects data source from server to autocomplete.
				 * @param  {Object} request  - Term property refers to value of input field.
				 * @param  {Object} response - Data suggested to the user, filtered 
				 * based on term property on the server-side.
				 * @return {[type]}          [description]
				 */
				source: function(request, response) {
					$.getJSON("/imagetracker/imageTracker/getTaggableUsers", {
						term: request.term
					}, function(data) {
						response($.map(data, function(item) {
							return {
								id: item.id,
								value: item.inputVal,
								renderVal: item.renderVal
							};
						}));
					});
				},
				/**
				 * Triggered when the field is blurred, if the value has changed.
				 * Resets input field if no item was selected from dropdown menu.
				 * @param  {Event} event
				 * @param  {Object} ui - Item selected from dropdown menu, otherwise null.
				 * @return {Boolean}
				 */
				change: function(event, ui) {
					// text value does not match any source data
					if (ui.item === null || ui.item === undefined) {
						input.val("");
						input[0].removeAttribute('data-user-id');
					}
					return false;
				},
				/**
				 * Triggered when an item is selected from the menu. 
				 * Replaces the input field's value and data attribute of user id 
				 * with the data of the selected item. 
				 * @param  {Event} event
				 * @param  {Object} ui - Item selected from dropdown menu, otherwise null.
				 * @return {Boolean}
				 */
				select: function(event, ui) {
					input.val(ui.item.value);
					input.attr('data-user-id', ui.item.id);
					return false;
				},
				/**
				 * Triggered when the menu is hidden. Input field is blurred.
				 * @param  {Event} event
				 * @param  {Object} ui - Item selected from dropdown menu, otherwise null.
				 * @return {Boolean}
				 */
				close: function(event, ui) {
					input.blur();
				}
			}).autocomplete("instance")._renderItem = function(ul, item) {
				// Create each option in the widget's menu with the data source property renderVal.
				return $("<li>")
					.append(item.renderVal)
					.appendTo(ul);
			};
		}

		return uiWidget;
	};

	/**
	 * Creates a rectangle with data attributes for coordinates.
	 * @param  {Number} x - X-coordinate for data attribute.
	 * @param  {Number} y - Y-coordinate for data attribute.
	 * @return {Object}   - HTMl element representing a rectangle.
	 */
	createRect = function(x, y) {
		var rect = $('<div></div>');
		rect.attr('class', 'rect');
		rect.attr('data-coord-x', x);
		rect.attr('data-coord-y', y);
		return rect;
	};

	/**
	 * Converts a mouse click coordinate to its corresponding coordinate on a scaled image.
	 * @param  {Number} clickCoord - Coordinate of a mouse click.
	 * @param  {Number} scale      - Scale factor of the image.
	 * @return {Number}            - Coordinates on scaled image as integer.
	 */
	convertClickCoordToImgCoord = function(clickCoord, scale) {
		return Math.round(clickCoord / scale);
	};

	/**
	 * Adds a function to the window.onload event without overwriting other functions.
	 * @param {Function} func - Function to be executed on the onload event.
	 */
	addLoadEvent = function(func) {
		var oldonload = window.onload;
		if (typeof window.onload != 'function') {
			window.onload = func;
		} else {
			window.onload = function() {
				if (oldonload) {
					oldonload();
				}
				func();
			};
		}
	};

	/**
	 * Save tags to the database. 
	 * @param  {Array} tags - Array of existing tags on image.
	 * @param  {Object} ids - Id's of current image (page_id, image_id, tag_id).  
	 * @return {Boolean}    - True, if ajax call was a success, otherwise false.
	 */
	save = function(tags, ids) {
		jQuery.ajax({
				url: '/imagetracker/imageTracker/save',
				data: {
					ids: ids,
					tags: tags
				}
			})
			.done(function(response) {
				return true;
			})
			.fail(function() {
				return false;
			});
	};

	/**
	 * Save existing tags to database when the save button is clicked.
	 * Set timeout to wait for change event on autocomplete ui widget, 
	 * otherwise change event is handled after the click and tag data is not updated.
	 * @return {Void}
	 */
	saveBtnListener = function() {
		$(".imgtracker-frame").on("click", '.imgtracker-save-btn', function(e) {
			//Wait for autocomplete change event to be handled and input field to be reset.
			setTimeout(function() {
				var frame, tagId, rects, img, tags, scaleX, ids;

				frame = $(e.delegateTarget);
				frame.toggleClass("editMode");

				tagId = e.delegateTarget.id.substring(11);
				rects = [];
				rects = $("#" + e.delegateTarget.id + " .rect");

				img = frame.find('img');

				scaleX = getImgScaleX(img);

				tags = [];

				ids = {
					tag_id: tagId,
					page_id: $(e.delegateTarget)[0].getAttribute('data-page-id'),
					image_id: img[0].getAttribute('data-img-id')
				};

				// save all tags with username into database table
				$.each(rects, function(key, value) {
					var input = $(this).find('input');
					input.prop('disabled', true);

					if (input[0].getAttribute('data-user-id')) {
						var tag = {};
						tag.user_id = input[0].getAttribute('data-user-id');
						tag.coord_x = value.getAttribute('data-coord-x');
						tag.coord_y = value.getAttribute('data-coord-y');
						// store px not %, so that only one side is needed *100/img[0].width
						tag.manual_width = $(this)[0].getAttribute('data-manual') ? (($(this)
							.width()) / scaleX) : null;
						tag.threshold = threshold;
						// delete tags with data-user-id but no value
						if (input.val() === "") {
							tag.remove = true;
						}
						tags.push(tag);
					}

					if (input.val() === "") {
						$(this).css('display', 'none');
					}
				});
				// save tags to database
				save(tags, ids);
			}, 250);
		});
	};

	/**
	 * Toggle edit mode when edit button is clicked.
	 * Edit mode: css class is added and all input fields are displayed.
	 * View mode: css class is removed and only tags with value are displayed on hover. 
	 * @return {Void}
	 */
	editBtnListener = function() {
		$(".imgtracker-frame").on("click", '.imgtracker-edit-btn', function(e) {
			var tagId, rects;

			$(e.delegateTarget).toggleClass("editMode");
			tagId = e.delegateTarget.id.substring(11);
			rects = [];
			rects = $("#" + e.delegateTarget.id + " .rect");

			$.each(rects, function(key, value) {
				$(this).css('display', 'inline');
				$(this).find('input').prop('disabled', false);
			});
		});
	};

	/**
	 * Remove all rectangles and display tags loaded from database when cancel button is clicked.
	 * @return {Void} 
	 */
	cancelBtnListener = function() {
		$(".imgtracker-frame").on("click", '.imgtracker-cancel-btn', function(e) {
			var tagId;
			$(e.delegateTarget).toggleClass("editMode");
			tagId = e.delegateTarget.id.substring(11);
			$("#" + e.delegateTarget.id + " .rect").remove();
			loadTags(tagId);
		});
	};

	/**
	 * Listen to delete button, which is only shown to users without tagging rights 
	 * on tags of themselves. Deletes tag of current user when delete button is 
	 * clicked and removes tag from database immediately.
	 * @return {Boolean} - True, if ajax call was a success, otherwise false.
	 */
	deleteBtnListener = function() {
		$(".imgtracker-frame").on("click", '.imgtracker-delete-btn', function(e) {
			var usertagId, parent;

			// remove tag
			parent = $(e.target.parentElement);
			parent.parent().remove();

			usertagId = e.target.getAttribute('data-usertag-id');

			// delete button
			jQuery.ajax({
					url: '/imagetracker/imageTracker/deleteTag',
					data: {
						usertagId: usertagId,
						userId: user.id
					}
				})
				.done(function(response) {
					return true;
				})
				.fail(function() {
					return false;
				});
		});
	};

	/**
	 * Initialize user with data of the currently logged in user and 
	 * add event listeners depending on tagging rights.
	 * @return {Boolean} True, if ajax call was a success, otherwise false.
	 */
	initUser = function() {
		jQuery.ajax({
				url: '/imagetracker/imageTracker/getUser',
			})
			.done(function(response) {
				user = JSON.parse(response);
				if (user.tagging_rights === '1') {
					// add full tagging functionality
					addLoadEvent(editBtnListener);
					addLoadEvent(saveBtnListener);
					addLoadEvent(cancelBtnListener);
					addLoadEvent(mouseImgListener);
				} else {
					// user can only delete tag of himself
					addLoadEvent(deleteBtnListener);
				}
				return true;
			})
			.fail(function() {
				return false;
			});
	};

	/**
	 * Listen to and handle mouse events on image in edit mode.
	 * @return {Void}
	 */
	mouseImgListener = function() {
		var isDrawing, offsetX, offsetY, startX, startY, mouseX, mouseY, rect;
		isDrawing = false;

		// e.which 1 = left mouse button
		$('.imgtracker').on('mousedown', function(e) {
			if (e.which === 1 && $(this).closest('.imgtracker-frame.editMode').length >
				0) {
				handleMouseDown(e);
			}
		}).on('mousemove', function(e) {
			if ($(this).closest('.imgtracker-frame.editMode').length > 0) {
				handleMouseMove(e);
			}

		}).on('mouseup', function(e) {
			if (e.which === 1 && $(this).closest('.imgtracker-frame.editMode').length >
				0) {
				handleMouseUp(e);
			}
		});

		/**
		 * End drawing mode and add empty input field to rectangle.
		 * @return {Void}
		 */
		function finishDrawing() {
			isDrawing = false;
			var input = createInput(null, true);
			rect.append(input);
		}

		/**
		 * If drawing mode is true, end drawing mode and create tag with input field.
		 * @param  {Event} e - Mouse up event
		 * @return {Void} 
		 */
		function handleMouseUp(e) {
			e.preventDefault();
			if (isDrawing) {
				finishDrawing();
			}
		}

		/**
		 * Draw rectangle in all directions if drawing mode is true.
		 * @param  {Event} e - Mouse move event.
		 * @return {Void}   
		 */
		function handleMouseMove(e) {

			switch (e.target.tagName.toLowerCase()) {
				case 'div':
					e.preventDefault();
					// enable making current rect smaller
					if (isDrawing && !$(e.target).is(rect)) {
						break;
					}
				case 'img':
					e.preventDefault();
					var img = $(e.currentTarget).find('img');

					if (isDrawing) {
						var offset_t, offset_l, left, top, side, newLeft, newTop, insideImg;

						offset_t = img.offset().top - $(window).scrollTop();
						offset_l = img.offset().left - $(window).scrollLeft();

						left = Math.round((e.clientX - offsetX));
						top = Math.round((e.clientY - offsetY));

						side = Math.abs(left - startX);
						newLeft = (left > startX) ? startX : left;
						newTop = (top > startY) ? startY : (startY - side);

						// insideImg = false if dimension of rect exceeds img
						insideImg = (newLeft < 0 || newTop < 0 || (newLeft + side) > img.width() || (newTop + side) > img.height()) ?
							false : true;
						// check if mouse is outside of img
						if (insideImg) {
							// Set dimension of rectangle; sides equal drawn width
							rect.css({
								'left': newLeft * 100 / img[0].width + '%',
								'top': newTop * 100 / img[0].height + '%',
								'width': side * 100 / img[0].width + '%',
								'height': side * 100 / img[0].height + '%'
							});
						}
					}
					break;
				default:
					// do nothing
			}
		}

		/**
		 * Starts the drawing of a manual tag when clicking on the image directly. 
		 * If already drawing, stops it and creates an input field for the newly drawn rectangle.
		 * @param  {Event} e - Mouse down event.
		 * @return {Void}   
		 */
		function handleMouseDown(e) {
			// if still drawing, mouse has had moved outside of image while drawing
			if (isDrawing) {
				finishDrawing();
			} else {
				switch (e.target.tagName.toLowerCase()) {
					case 'img':
						e.preventDefault();

						var img, offset_t, offset_l, left, top, scaleX, scaleY, x, y, tagId, deleteRect, side, newLeft, newTop;
						isDrawing = true;

						img = $(e.target);
						offset_t = img.offset().top - $(window).scrollTop();
						offset_l = img.offset().left - $(window).scrollLeft();

						offsetX = offset_l;
						offsetY = offset_t;
						left = Math.round((e.clientX - offset_l));
						top = Math.round((e.clientY - offset_t));

						startX = left;
						startY = top;

						scaleX = getImgScaleX(img);
						scaleY = getImgScaleY(img);

						x = convertClickCoordToImgCoord(left, scaleX);
						y = convertClickCoordToImgCoord(top, scaleY);

						// add manual tag to DOM
						tagId = e.delegateTarget.id.substring(11);
						rect = createRect(x, y);
						rect.attr('data-manual', true);
						deleteRect = $('<button type="button" class="rm-manual">x</button>');

						deleteRect.on('click', function(e) {
							$(this).parent().remove();
						});

						rect.append(deleteRect);
						// insert after image
						rect.insertAfter(img[0]);

						side = Math.abs(left - startX);
						newLeft = (left > startX) ? startX : left;
						newTop = (top > startY) ? startY : top;

						// Set dimension of rectangle
						rect.css({
							'left': newLeft * 100 / img[0].width + '%',
							'top': newTop * 100 / img[0].height + '%',
							'width': side * 100 / img[0].width + '%',
							'height': side * 100 / img[0].height + '%'
						});
						break;
					default:
						// do nothing
				}
			} // end of if isDrawing
		}
	};

	// anything returned becomes public
	return {
		loadTags: loadTags,
		initUser: initUser
	};

}();

Imagetracker.Module.initUser();