<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 * ImageTracker controller class file.
 */
class ImageTrackerController extends BaseController
{
    /**
     * Get all users that allow tagging of themselves.
     * Formats user data to match configured template strings.
     *
     * e.g. '{{lastname}}, {{firstname}}' --> Mustermann, Max
     *
     * @return array of users
     */
    public function actionGetTaggableUsers() {
        $filteredUsersArr = array();
        $term = Yii::app()->request->getParam('term',''); 

        if($term != ''){
            $keywords = preg_split( "/ /", $term );

            $q = new CDbCriteria();
            foreach ( $keywords as $keyword ) {
                $qtmp = new CDbCriteria();
                // remove special characters from keyword
                $keyword = preg_replace( "/[^a-zA-Z0-9.@-]/", "", $keyword );
                $qtmp->addSearchCondition( 'profile.firstname', $keyword, true, "OR" );
                $qtmp->addSearchCondition( 'profile.lastname', $keyword, true, "OR" );
                $qtmp->addSearchCondition( 'email', $keyword, true, "OR" );
                $q->mergeWith( $qtmp, "AND" );
            }

            $q->limit = 10;

            $taggableUsers = User::model()->with( array(
                    'profile' => array(
                        'condition' => 'tagging_allowed = :tagging_allowed',
                        'params' => array(
                            'tagging_allowed' => 1
                        )
                    )
                ) )->findAll( $q );

            if ( $taggableUsers ) {
                $renderTemplateStr = Yii::app()->getModule( 'imagetracker' )->renderTemplateStr;
                $inputTemplateStr = Yii::app()->getModule( 'imagetracker' )->inputTemplateStr;
                foreach ( $taggableUsers as $user ) {
                    $userArr = array();
                    $userArr['id'] = $user->id;

                    $templateValues = array();
                    $templateValues['username'] = $user->username; //assign only some columns not entire $client object.
                    $templateValues['email'] = $user->email;
                    $templateValues['firstname'] = $user->profile->firstname;
                    $templateValues['lastname'] = $user->profile->lastname;
                    // populate string for autocomplete list item
                    $userArr['renderVal'] = $this->getStringFromTemplate( $templateValues, $renderTemplateStr );
                    // populate value for input field
                    $userArr['inputVal'] = $this->getStringFromTemplate( $templateValues, $inputTemplateStr );

                    // string needle found in haystack
                    $filteredUsersArr[] = $userArr;

                }
            }
        }

        echo CJavaScript::jsonEncode( $filteredUsersArr );

        return true;
    }

    /**
     * Populate template string with array values.
     *
     * @param array $array of user data
     * @param string $tmpString
     * @return string
     */
    private function getStringFromTemplate( $array, $tmpString ) {
        $inputVal = preg_replace_callback( '/{{(\w+)}}/', function( $match ) use( $array ) {
                return $array[$match[1]];
            }, $tmpString );
        return $inputVal;
    }

    /**
     * Get all tags inside a tagtype imagetracker. Used on first page load.
     *
     * @return array of users
     */
    public function actionGetTagsByTagId() {
        $tagId = Yii::app()->request->getParam('tagId',''); 

        if($tagId != ''){

            $tags = ImageTrackerTag::model()->with( array(
                    'profile'=>array(
                        'joinType'=>'INNER JOIN',
                    ),
                ) )->findAllByAttributes( array( 'tag_id'=>$tagId ) );

            // populate value for input field
            $inputTemplateStr = Yii::app()->getModule( 'imagetracker' )->inputTemplateStr;

            // Map to Array with related data
            $tagsArr = array();

            if ( $tags ) {
                foreach ( $tags as $tag ) {
                    $newtag = array();
                    $newtag['id'] = $tag->id;
                    $newtag['user_id'] = $tag->user_id;
                    $newtag['coord_x'] = $tag->coord_x; 
                    $newtag['coord_y'] = $tag->coord_y;
                    $newtag['manual_width'] = $tag->manual_width;

                    // populate value for input field
                    $templateValues = array();
                    $templateValues['username'] = $tag->user->username; 
                    $templateValues['email'] = $tag->user->email;
                    $templateValues['firstname'] = $tag->profile->firstname;
                    $templateValues['lastname'] = $tag->profile->lastname;
                    $newtag['value'] = $this->getStringFromTemplate( $templateValues, $inputTemplateStr );

                    $tagsArr[] = $newtag;
                }
            }
            echo CJavaScript::jsonEncode( $tagsArr );
            return true;
        } else {
            return false;
        }
    }

    /**
     * Save tags.
     *
     * @return array of users
     */
    public function actionSave() {
        if ( !Yii::app()->user->isGuest && $this->module->user->profile->tagging_rights ) {

            $ids = Yii::app()->request->getParam('ids',''); 
            $tags = Yii::app()->request->getParam('tags', array());

            if($ids != ''){
                if ( count( $tags ) > 0 ) {
                    foreach ( $tags as $tagOptions ) {
                        $tagOptions['page_id'] = $ids['page_id'];
                        $tagOptions['image_id'] = $ids['image_id'];
                        $tagOptions['tag_id'] = $ids['tag_id'];
                        $this->saveTag( $tagOptions );
                    }
                }

                // delete (manual) tags that have been completely removed in the frontend
                $dbTags = ImageTrackerTag::model()->findAllByAttributes( array(
                        'tag_id' => $ids['tag_id'],
                        'page_id' => $ids['page_id'],
                        'image_id' => $ids['image_id']
                    ) );

                // no tags, delete all remaining tags in db
                if ( count( $tags ) == 0 && count( $dbTags ) > 0 ) {
                    foreach ( $dbTags as $dbTag ) {
                        $dbTag->delete();
                    }
                } else {
                    // Delete db-Tags, that have not been submitted during save
                    foreach ( $dbTags as $key => $dbTag ) {
                        $exists = false;
                        foreach ( $tags as $tag ) {
                            if ( ( $dbTag->coord_x == $tag['coord_x'] ) && ( $dbTag->coord_y == $tag['coord_y'] ) ) {
                                // tag exists in db
                                $exists = true;
                                break;
                            }
                        }
                        // delete unused tag
                        if ( $exists == false ) {
                            $dbTag->delete();
                        }
                    } // end foreach
                }
            } // end if else $ids set
        } // end if isGuest
    }

    /**
     * Create and save new tag or update and save existing tag.
     * @param  array $options of tag
     * @return void
     */
    private function saveTag( $options ) {
        $coordX = $options['coord_x'];
        $coordY = $options['coord_y'];
        $threshold = $options['threshold'];

        // tracked coordinates can vary by a few pixels in different browsers,
        // thus a threshold is needed for the search
        $q = new CDbCriteria();
        $q->addBetweenCondition( 'coord_x', $coordX-$threshold, $coordX+$threshold );
        $q->addBetweenCondition( 'coord_y', $coordY-$threshold, $coordY+$threshold );

        $tag = ImageTrackerTag::model()->findByAttributes( array(
                'tag_id' => $options['tag_id'],
                'page_id' => $options['page_id'],
                'image_id' => $options['image_id']
            ), $q );

        // if tag doesn't exist yet, create new one
        if ( !$tag ) {
            // create new tag
            $tag = new ImageTrackerTag();
        } else {
            if ( array_key_exists( 'remove', $options ) ) {
                if($tag->delete()){
                    return true;
                };
            }
        }

        $tag->image_id      = $options['image_id'];
        $tag->tag_id        = $options['tag_id'];
        $tag->page_id       = $options['page_id'];
        $tag->user_id       = $options['user_id'];
        $tag->coord_x       = $options['coord_x'];
        $tag->coord_y       = $options['coord_y'];
        $tag->manual_width  = $options['manual_width'];
        $tag->tagged_by     = Yii::app()->user->id;

        $tag->validate();
        $tag->save();

        if ( $tag->hasErrors() ) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Delete tag of current user.
     * User can only delete tag of himself.
     *
     * @return void
     */
    public function actionDeleteTag() {
        if ( !Yii::app()->user->isGuest ) {

            $usertagId = Yii::app()->request->getParam('usertagId',''); 
            $userId = Yii::app()->request->getParam('userId',''); 

            if ( $userId != '' && $usertagId != '' && $this->module->user->id == $userId ) {
                $tag = ImageTrackerTag::model()->findByAttributes( array(
                        'id' => $usertagId,
                        'user_id' => $userId
                    ) );

                if ( $tag ) {
                    $tag->delete();
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * Get currently logged in user
     *
     * @return array of users
     */
    public function actionGetUser() {
        if ( !Yii::app()->user->isGuest ) {
            if ( $this->module->user ) {
                $userArr = array();
                $userArr['id'] = $this->module->user->id;
                $userArr['tagging_allowed'] = $this->module->user->profile->tagging_allowed;
                $userArr['tagging_rights'] = $this->module->user->profile->tagging_rights;

                echo CJavaScript::jsonEncode( $userArr );
            }
        } else {
            echo CJavaScript::jsonEncode( array() );
        }
        return true;
    }
}
