<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 */
class Release extends CComponent
{
    //folder where release will be saved
    public static $temporaryDir = '/tmp';

    private $_zipLocation;

    private $_currentVersion;

    private $_invitationMessage;

    private $_finishMessage;

    private $_zipArchive;

    /**
     * Constructor
     *
     * @param string $zipLocation location of release zip archive
     */
    public function __construct($zipLocation)
    {
        $this->_zipLocation = $zipLocation;
    }

    /**
     * Public getter
     *
     * @return string
     */
    public function getZipLocation()
    {
        return $this->_zipLocation;
    }

    /**
     * Open zip archive
     *
     * @throws Exception
     *
     * @return mixed
     */
    private function _getZipArchive()
    {
        if ($this->_zipArchive === null) {
            $this->_zipArchive = new ZipArchive();
            if (($errorCode = $this->_zipArchive->open($this->_zipLocation, ZIPARCHIVE::CHECKCONS)) !== true) {
                throw new Exception("Invalid zip file. Error: $errorCode");
            }
        }

        return $this->_zipArchive;
    }

    /**
     * Retrieve release version from archive
     *
     * @return mixed
     */
    public function getVersion()
    {
        if ($this->_currentVersion === null) {
            $zip = $this->_getZipArchive();
            $tmpFile = Yii::getPathOfAlias('common.runtime.release_main_config');
            file_put_contents($tmpFile, $zip->getFromName('frontend/config/main.php'));
            $updatesConfig = include $tmpFile;
            $this->_currentVersion = $updatesConfig['modules']['updates']['currentVersion'];
            unlink($tmpFile);
        }
        return $this->_currentVersion;
    }

    /**
     * Retrieve invitation message from archive
     *
     * @return mixed
     */
    public function getInvitationMessage()
    {
        if ($this->_invitationMessage === null) {
            $zip = $this->_getZipArchive();
            $this->_invitationMessage = $zip->getFromName('common/runtime/invitationMessage');
        }

        return $this->_invitationMessage;
    }

    /**
     * Retrieve finish message from archive
     *
     * @return mixed
     */
    public function getFinishMessage()
    {
        if ($this->_finishMessage === null) {
            $zip = $this->_getZipArchive();
            $this->_finishMessage = $zip->getFromName('common/runtime/finishMessage');
        }

        return $this->_finishMessage;
    }


    /**
     * Update project placed in $destination folder from release archive
     *
     * @param string $destination destination folder
     *
     * @return void
     */
    public function update($destination)
    {
        ZipHelper::extract($this->getZipLocation(), realpath($destination));
        unlink($destination . '/common/runtime/invitationMessage');
        unlink($destination . '/common/runtime/finishMessage');
    }

    /**
     * Create release of current project. Compress all necessary data into archive
     *
     * @param string $location          release folder
     * @param array  $exclude           files/folder which need to be excluded
     * @param string $invitationMessage message which will be displayed before update process
     * @param string $finishMessage     message which will be displayed after update process
     *
     * @return Release
     */
    public static function make($location, $exclude = array(), $invitationMessage = '', $finishMessage = '')
    {
        $zipPath = self::$temporaryDir . '/' . self::releaseName();
        @unlink($zipPath);
        ZipHelper::compress(
            array(
                array(
                    'path' => $location,
                    'exclude' => $exclude
                ),
                'common/runtime/invitationMessage' => $invitationMessage,
                'common/runtime/finishMessage' => $finishMessage
            ), $zipPath
        );

        return new Release($zipPath);
    }


    /**
     * Returns release name
     *
     * @static
     *
     * @return string
     */
    public static function releaseName()
    {
        $date = date('Y_m_d');
        return "release_{$date}_v" . Yii::app()->getModule('updates')->currentVersion . ".zip";
    }

    /**
     * Runs migrate command
     *
     * @static
     *
     * @return void
     */
    public static function migrate()
    {
        //Yii::import('system.cli.commands.MigrateCommand');
        //$command = new MigrateCommand('', new CConsoleCommandRunner());
        //$command->actionUp(array());
    }


    /**
     * Backup project folder
     *
     * @param string $destination folder where backup will be saved
     *
     * @return bool
     */
    public static function backupProject($destination = '/tmp')
    {
        $date = date('Y-m-d');
        $zipPath = $destination . '/' . "gentics_portal_node_php_{$date}_v" . Yii::app()->getModule('updates')->currentVersion . '.bak.zip';
        @unlink($zipPath);
        $dbDumper = new DbDumper();
        $dump = $dbDumper->getDump();
        ZipHelper::compress(
            array(
                array(
                    'path' => Yii::getPathOfAlias('site'),
                    'exclude' => array(
                        'common/runtime',
                        'custom/runtime',
                        'frontend/runtime',
                        'frontend/www/assets'
                    )
                ),
                "db_dump_$date.sql" => $dump
            ),
            $zipPath
        );

        return $zipPath;
    }
}
